@@ -1,7 +1,7 @@
 # ImageThought.py
 # This file is part of labyrinth
 #
-# Copyright (C) 2006 - Don Scorgie <DonScorgie@Blueyonder.co.uk>
+# Copyright (C) 2006 - Don Scorgie <Don@Scorgie.org>
 #
 # labyrinth is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ import xml.dom.minidom as dom
 import xml.dom
 import gettext
 _ = gettext.gettext
+import cairo
 
 import BaseThought
 import utils
@@ -36,8 +37,8 @@ MODE_DRAW = 2
 UNDO_RESIZE = 0
 
 class ImageThought (BaseThought.ResizableThought):
-	def __init__ (self, coords, pango_context, thought_number, save, undo, loading):
-		super (ImageThought, self).__init__(save, "image_thought", undo)
+	def __init__ (self, coords, pango_context, thought_number, save, undo, loading, background_color):
+		super (ImageThought, self).__init__(save, "image_thought", undo, background_color, None)
 
 		self.identity = thought_number
 		margin = utils.margin_required (utils.STYLE_NORMAL)
@@ -72,11 +73,19 @@ class ImageThought (BaseThought.ResizableThought):
 				fname = dialog.get_filename()
 		else:
 			fname = filename
-		self.filename = fname
+
 		try:
 			self.orig_pic = gtk.gdk.pixbuf_new_from_file (fname)
 		except:
-			return False
+			try:
+				# lets see if file was imported and is already extracted
+				fname = utils.get_save_dir() + 'images/' + utils.strip_path_from_file_name(filename)
+				self.orig_pic = gtk.gdk.pixbuf_new_from_file (fname)
+			except:
+				return False
+
+		self.filename = fname
+				
 		if not filename:
 			self.width = self.orig_pic.get_width ()
 			self.height = self.orig_pic.get_height ()
@@ -88,7 +97,10 @@ class ImageThought (BaseThought.ResizableThought):
 		return True
 
 	def draw (self, context):
-		utils.draw_thought_outline (context, self.ul, self.lr, self.am_selected, self.am_primary, utils.STYLE_NORMAL)
+		if len (self.extended_buffer.get_text()) == 0:
+			utils.draw_thought_outline (context, self.ul, self.lr, self.background_color, self.am_selected, self.am_primary, utils.STYLE_NORMAL)
+		else:
+			utils.draw_thought_outline (context, self.ul, self.lr, self.background_color, self.am_selected, self.am_primary, utils.STYLE_EXTENDED_CONTENT)
 
 		if self.pic:
 			context.set_source_pixbuf (self.pic, self.pic_location[0], self.pic_location[1])
@@ -97,13 +109,16 @@ class ImageThought (BaseThought.ResizableThought):
 		context.set_source_rgb (0,0,0)
 
 	def export (self, context, move_x, move_y):
-		utils.export_thought_outline (context, self.ul, self.lr, self.am_selected, self.am_primary, utils.STYLE_NORMAL,
+		utils.export_thought_outline (context, self.ul, self.lr, self.background_color, self.am_selected, self.am_primary, utils.STYLE_NORMAL,
 									  (move_x, move_y))
-
 		if self.pic:
-			context.set_source_pixbuf (self.pic, self.pic_location[0]+move_x, self.pic_location[1]+move_y)
-			context.rectangle (self.pic_location[0]+move_x, self.pic_location[1]+move_y,
-							   self.width, self.height)
+			if hasattr(context, "set_source_pixbuf"):
+				context.set_source_pixbuf (self.pic, self.pic_location[0]+move_x, self.pic_location[1]+move_y)
+			elif hasattr(context, "set_source_surface"):
+				pixel_array = utils.pixbuf_to_cairo (self.pic.get_pixels_array())
+				image_surface = cairo.ImageSurface.create_for_data(pixel_array, cairo.FORMAT_ARGB32, self.width, self.height, -1)
+				context.set_source_surface (image_surface, self.pic_location[0]+move_x, self.pic_location[1]+move_y)
+			context.rectangle (self.pic_location[0]+move_x, self.pic_location[1]+move_y, self.width, self.height)
 			context.fill ()
 		context.set_source_rgb (0,0,0)
 
@@ -130,7 +145,7 @@ class ImageThought (BaseThought.ResizableThought):
 		self.emit ("update_view")
 		self.undo.unblock ()
 
-	def process_button_down (self, event, mode):
+	def process_button_down (self, event, mode, transformed):
 		modifiers = gtk.accelerator_get_default_mod_mask ()
 		self.button_down = True
 		if event.button == 1:
@@ -146,7 +161,7 @@ class ImageThought (BaseThought.ResizableThought):
 		self.emit ("update_view")
 
 
-	def process_button_release (self, event, unending_link, mode):
+	def process_button_release (self, event, unending_link, mode, transformed):
 		self.button_down = False
 		if unending_link:
 			unending_link.set_child (self)
@@ -159,7 +174,7 @@ class ImageThought (BaseThought.ResizableThought):
 														self.orig_size, (self.ul, self.width, self.height)))
 			self.want_move = False
 
-	def handle_motion (self, event, mode):
+	def handle_motion (self, event, mode, transformed):
 		if self.resizing == self.RESIZE_NONE or not self.want_move or not event.state & gtk.gdk.BUTTON1_MASK:
 			if not event.state & gtk.gdk.BUTTON1_MASK:
 				return False
@@ -167,10 +182,10 @@ class ImageThought (BaseThought.ResizableThought):
 				self.emit ("create_link", \
 				 (self.ul[0]-((self.ul[0]-self.lr[0]) / 2.), self.ul[1]-((self.ul[1]-self.lr[1]) / 2.)))
 			return True
-		diffx = event.x - self.motion_coords[0]
-		diffy = event.y - self.motion_coords[1]
+		diffx = transformed[0] - self.motion_coords[0]
+		diffy = transformed[1] - self.motion_coords[1]
 		tmp = self.motion_coords
-		self.motion_coords = (event.x,event.y)
+		self.motion_coords = transformed
 		if self.resizing == self.RESIZE_LEFT:
 			if self.width - diffx < 10:
 				self.motion_coords = tmp
@@ -242,12 +257,16 @@ class ImageThought (BaseThought.ResizableThought):
 	def update_save (self):
 		text = self.extended_buffer.get_text ()
 		if text:
-			self.extended_element.replaceWholeText (text)
+			self.extended_buffer.update_save()
 		else:
-			self.extended_element.replaceWholeText ("LABYRINTH_AUTOGEN_TEXT_REMOVE")
+			try:
+				self.element.removeChild(self.extended_buffer.element)
+			except xml.dom.NotFoundErr:
+				pass
 		self.element.setAttribute ("ul-coords", str(self.ul))
 		self.element.setAttribute ("lr-coords", str(self.lr))
 		self.element.setAttribute ("identity", str(self.identity))
+		self.element.setAttribute ("background-color", self.background_color.to_string())
 		self.element.setAttribute ("file", str(self.filename))
 		self.element.setAttribute ("image_width", str(self.width))
 		self.element.setAttribute ("image_height", str(self.height))
@@ -274,6 +293,11 @@ class ImageThought (BaseThought.ResizableThought):
 		self.lr = utils.parse_coords (tmp)
 		self.filename = node.getAttribute ("file")
 		self.identity = int (node.getAttribute ("identity"))
+		try:
+			tmp = node.getAttribute ("background-color")
+			self.background_color = gtk.gdk.color_parse(tmp)
+		except ValueError:
+			pass
 		self.width = float(node.getAttribute ("image_width"))
 		self.height = float(node.getAttribute ("image_height"))
 		if node.hasAttribute ("current_root"):
@@ -287,11 +311,7 @@ class ImageThought (BaseThought.ResizableThought):
 
 		for n in node.childNodes:
 			if n.nodeName == "Extended":
-				for m in n.childNodes:
-					if m.nodeType == m.TEXT_NODE:
-						text = m.data
-						if text != "LABYRINTH_AUTOGEN_TEXT_REMOVE":
-							self.extended_buffer.set_text (text)
+				self.extended_buffer.load(n)
 			else:
 				print "Unknown: "+n.nodeName
 		margin = utils.margin_required (utils.STYLE_NORMAL)
